/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import java.util.LinkedList;
import java.util.List;

/**
 * Strategy gather all queries into list.
 *
 * @see SqlCallback
 * @see Sql
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class ListParser implements SqlCallback
{
    /**
     * here will be placed all SQL commands parsed by parser
     * without any processing.
     */
    private final List<String> queries;

    /**
     * Constructor has package-visibility because the
     * object is constructed inside {@link SqlEngine}
     */
    ListParser()
    {
        this.queries = new LinkedList<String>();
    }


    /**
     * just add the SQL command into internal list
     * @param sql
     */
    @Override
    public final void doSql(String sql)
    {
        queries.add(sql);
    }


    /**
     * return all parsed SQL commands/queries.
     * @return
     */
    public final List<String> andReturnListOfQueries()
    {
        return queries;
    }
}
