/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx;

import com.celum.nyx.interceptor.SqlInterceptor;
import org.antlr.v4.runtime.tree.TerminalNode;

import java.util.List;

/**
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
class ParserGrammarListener extends AbstractSqlGrammarListener {

    private final List<SqlInterceptor> interceptorChain;
    private final SqlCallback strategy;
    private String actualScript = "";
    private boolean isScript = false;


    /**
     * Constructor
     * @param strategy
     */
    ParserGrammarListener(SqlCallback strategy, List<SqlInterceptor> interceptorChain)
    {
        this.strategy = strategy;
        this.interceptorChain = interceptorChain;
    }

    @Override
    public void exitSqlCommand(SqlGrammarParser.SqlCommandContext ctx)
    {
        if (!isScript) {
            finishScript();
        }
    }

    @Override
    public void exitEndScript(SqlGrammarParser.EndScriptContext ctx)
    {
        finishScript();
    }

    @Override
    public void enterBeginScript(SqlGrammarParser.BeginScriptContext ctx)
    {
        isScript = true;
    }

    @Override
    public void visitTerminal(TerminalNode terminalNode)
    {
        String txt = terminalNode.getSymbol().getText();
        if (txt == null) {
            return;
        }
        actualScript = actualScript + " " +terminalNode.getSymbol().getText();
    }


    /**
     * This is the main method where is each SQL command executed.
     */
    protected void finishScript()
    {
        String sqlToExecute = actualScript;
        actualScript = "";
        isScript = false;

        //prepare SQL, check whether is not null or empty etc
        if (sqlToExecute == null) {
            return;
        }

        sqlToExecute = sqlToExecute.trim();
        if (sqlToExecute.isEmpty()) {
            return;
        }

        if (sqlToExecute.endsWith("/")) {
            sqlToExecute = sqlToExecute.substring(0, sqlToExecute.length() - 1);
        } else if (sqlToExecute.endsWith(";")) {
            sqlToExecute = sqlToExecute.substring(0, sqlToExecute.length() - 1);
        }

        //do pre-handling of SQL via interceptors
        for (SqlInterceptor i : interceptorChain) {
            sqlToExecute = i.preHandleSql(sqlToExecute);
        }

        if (sqlToExecute != null && !sqlToExecute.isEmpty()) {
            strategy.doSql(sqlToExecute);
        }

        //do post-handling of SQL via interceptors
        for (SqlInterceptor i : interceptorChain) {
            i.afterCompleteSql(sqlToExecute);
        }

    }
}
