package com.celum.nyx;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 *
 */
public class PreparedStatementQueryCallback implements SqlCallback, SqlCloseable
{
    private final DataSource ds;
    private final ValuesSetter setter;

    /**
     * flag indicates and ensure the doSql() is performed only once.
     * It's because this callback is dedicated only to one-liner SQLs,
     * not the SQL scripts.
     */
    private boolean notYetRun = true;


    /**
     * reference to prepared statement that is created during 'doSql' call.
     * and will be closed by SqlResultSet.
     */
    private Connection connection;

    /**
     * reference to prepared statement that is created during 'doSql' call.
     * and will be closed by SqlResultSet.
     */
    private PreparedStatement pstmt;

    /**
     * reference to prepared statement that is created during 'doSql' call
     * and will be closed by SqlResultSet.
     */
    private ResultSet rs;

    /**
     * Constructor
     * @param ds
     */
    public PreparedStatementQueryCallback(DataSource ds, ValuesSetter setter)
    {
        this.ds = ds;
        this.setter = setter;
    }


    /**
     * perform SQL Query with PreparedStatement
     */
    @Override
    public void doSql(String sql)
    {
        if (!notYetRun) {
            throw new DataAccessException("Invalid SQL. The SQL must be simple command, not a whole script.");
        }
        notYetRun = false;

        try {
            connection = ds.getConnection();
            pstmt = connection.prepareStatement(sql);
            setter.setValues(pstmt);
            rs = pstmt.executeQuery();
        } catch (SQLException e) {
            close();
            throw new DataAccessException("SQL error " + sql, e);
        }
    }

    /**
     * returns the wrapped result set which provide operations over result set.
     */
    SqlResultSet andGetResultSet()
    {
        return new SqlResultSet(this, rs);
    }


    @Override
    public void close()
    {
        JDBCUtils.closeQuietly(connection, pstmt, rs);
    }
}
