package com.celum.nyx;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
 *
 */
public class PreparedStatementUpdateCallback implements SqlCallback
{
    private final DataSource ds;
    private final PreparedStatementSetter setter;
    private boolean notYetRun = true;

    public PreparedStatementUpdateCallback(DataSource ds, PreparedStatementSetter setter)
    {
        this.ds = ds;
        this.setter = setter;
    }

    @Override
    public void doSql(String sql)
    {
        if (!notYetRun) {
            throw new DataAccessException("Invalid SQL. The SQL must be simple command, not a whole script.");
        }

        notYetRun = false;
        Connection connection = null;

        try {
            connection = ds.getConnection();
            PreparedStatement pstmt = connection.prepareStatement(sql);

            setter.setValues(pstmt);
            if (setter instanceof BatchSetter) {
                pstmt.executeBatch();
            } else if (setter instanceof ValuesSetter) {
                pstmt.execute();
            }

        } catch (SQLException e) {
            throw new DataAccessException("SQL error " + sql, e);
        } finally {
            JDBCUtils.closeQuietly(connection);
        }
    }
}
