package com.celum.nyx;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

/**
 *
 */
class StatementQueryCallback implements SqlCallback, SqlCloseable {

    /**
     * holds datasource that comes from outside (mostly from SqlEngine)
     */
    private final DataSource ds;

    /**
     * flag ensure the doSql() is performed only once.
     * It's because this callback is dedicated only to one-liner SQLs,
     * not the SQL scripts.
     */
    private boolean firstRun = true;


    /**
     * reference to connection that is created during 'doSql' call.
     * and will be closed by SqlResultSet.
     */
    private Connection connection;

    /**
     * reference to statement that is created during 'doSql' call.
     * and will be closed by SqlResultSet.
     */
    private Statement stmt;


    /**
     * reference to result set that is created during 'doSql' call.
     * and will be closed by SqlResultSet.
     */
    private ResultSet rs;


    /**
     * Constructor
     * @param ds
     */
    StatementQueryCallback(DataSource ds)
    {
        this.ds = ds;
    }

    @Override
    public void doSql(String sql)
    {
        //first run check
        if (!firstRun) {
            throw new DataAccessException("Invalid SQL. The SQL must be simple command, not a whole script.");
        } else {
            firstRun = false;
        }

        //perform the SQL
        try {
            connection = ds.getConnection();
            stmt = connection.createStatement();
            rs = stmt.executeQuery(sql);
        } catch (SQLException e) {
            close();
            throw new DataAccessException("SQL error " + sql, e);
        }
    }


    SqlResultSet andGetResultSet()
    {
        return new SqlResultSet(this, rs);
    }


    @Override
    public void close()
    {
        JDBCUtils.closeQuietly(connection, stmt, rs);
    }
}
