/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx.interceptor;

/**
 * Interceptors could affect the SQL command
 * before is executed. They're stored in {@link com.celum.nyx.Sql}
 * in interceptor chain.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public interface SqlInterceptor {

    /**
     * Here you will implement your intercepting code that
     * could modify the SQL command.
     *
     * @param sql incomming SQL
     * @return SQL that is then executed (or moved into next interceptor in chain)
     */
    public String preHandleSql(String sql);


    /**
     * method is invoked when SQL command has been executed successfully
     * @param sql
     */
    public void afterCompleteSql(String sql);
}
