/*****************************************************************************
 * Copyright 2013 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.nyx.mapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

/**
 * Base class for all row mappers that is mapping rows
 * into list of objects.
 */
public abstract class ListMapper<T> implements RowMapper {

    /** list where are stored mapped rows */
    private List<T> list;


    /**
     * Constructor
     */
    public ListMapper()
    {
        this.list = new LinkedList<T>();
    }


    /**
     * do mapping of rows where result is added
     * into list.
     *
     * @param rs
     * @param rownum
     */
    @Override
    public final void map(ResultSet rs, long rownum) throws SQLException
    {
        T result = mapAs(rs, rownum);
        if (result != null) {
            list.add(result);
        }
    }


    /**
     * return the list of mapped rows
     */
    public final List<T> andReturnList()
    {
        return list;
    }


    /**
     * here you will implement you mapping code that return
     * mapped row to object as result which will be added into
     * list.
     */
    protected abstract T mapAs(ResultSet rs, long rownum) throws SQLException;
}
